<?php

/*
 * Private Message Window
 * This Plugin is a extention for the command "/pm player msg" which allows it to
 * select the player in a window instead of chat. Currently only TMF supported.
 * Just type "/pmx your private message for an other player", then press enter
 * and select the user to send the message to.
 * ----------------------------------------------------------------------------------
 * Author:	undef.de
 * Version:	0.8.7
 * Date:	2010-10-03
 * Copyright:	2009 - 2010 by undef.de
 * Home:	http://www.undef.de/Trackmania/Plugins/
 * System:	xaseco/1.06 and up
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: requires plugin.rasp_chat.php (and that requires chat.admin.php)
 */

/* The following manialink id's are used in this plugin (the 913 part of id can be changed on trouble):
 * 913nnnn	id for action, nnnn = PlayerID
 */

Aseco::registerEvent('onStartup',			'pmx_onStartup');
Aseco::registerEvent('onPlayerManialinkPageAnswer',	'pmx_onPlayerManialinkPageAnswer');
Aseco::registerEvent('onPlayerConnect',			'pmx_onPlayerConnect');

Aseco::addChatCommand('pmx',				'Send a private message to a player, see /pmx help');

global $pmx_config, $PMX_VERSION;

// Logs all user actions, set to 'true' or 'false'
$pmx_config['LOG_USER_ACTION'] = true;

// DO NOT CHANGE THIS!!!
$pmx_config['MANIALINK_ID'] = '913';
$PMX_VERSION = '0.8.7';

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function pmx_onStartup ($aseco) {
	if ( version_compare(XASECO_VERSION, 1.06, '<') ) {
		trigger_error('[plugin.private_message_window.php] Not supported xaseco version ('. XASECO_VERSION .')! Please update to min. version 1.06!', E_USER_ERROR);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function chat_pmx ($aseco, $command) {
	global $pmx_config, $PMX_VERSION;

	if ($command['params'] == '') {
		$message = "{#server}> No message given! See {#highlite}/pmx help{#server}";
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
		return;
	}
	elseif (strtoupper($command['params']) == 'HELP') {
		$message = "{#server}> Type {#highlite}/pmx message{#server} to send a private message and select in the Popup Window the player to send the message.";
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
		return;
	}
	else {
		// Save the given message
		$command['author']->data['PMX_MESSAGE'] = $command['params'];
	}

	// Players there?
	if (count($aseco->server->players->player_list) <= 1) {
		$message = "{#server}> No players found to send a message to! You're alone ;)";
		$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
		return;
	}


	// Set header of PopUp-Window
	$header = '$L[http://www.undef.de/Trackmania/Plugins/]Private Message Window Plugin/'. $PMX_VERSION .'$L';

	// Initialisations
	$command['author']->msgs = array();
	$command['author']->msgs[0] = array(1, $header, array(1.4, 0.8, 0.4, 0.2), array('Icons64x64_1', 'Outbox'));


	// Create the Recent-Players-List
	$lines = 0;
	$title[0] = array('$FF0Recent Players (select player to send your private message)', '', '');
	$title[1] = array('Nickname', 'Login', 'Status');
	$recent_players = array();
	$recent_players = array_merge($title);
	if (count($command['author']->data['PMX_RECENT']) > 0) {
		foreach ($command['author']->data['PMX_RECENT'] as $recent_login) {
			foreach ($aseco->server->players->player_list as $player) {
				if ($player->login == $recent_login) {
					$recent_players[] = array(
						array(pmx_handleNicknames($player->nickname), $pmx_config['MANIALINK_ID'].$player->id),
						'$n'. $player->login,
						$player->isspectator ? 'Spectator' : 'Player'
					);

					$lines++;
					if ($lines > 12) {
						$command['author']->msgs[] = $recent_players;
						$lines = 0;
						$recent_players = array();
						$recent_players = array_merge($title);
					}
				}
			}
		}
	}
	// Add if last batch exists
	if (count($recent_players) > 2) {	// two header lines
		$command['author']->msgs[] = $recent_players;
	}
	unset($recent_players);



	// Create the All-Players-List
	$player_count = 0;
	$player_list = array();
	$lines = 0;
	$title[0] = array('$0D3All Players (select player to send your private message)', '', '');
	$title[1] = array('Nickname', 'Login', 'Status');
	$all_players = array();
	$all_players = array_merge($title);
	foreach ($aseco->server->players->player_list as $player) {
		// Hide recents from all playerlist
		if (count($command['author']->data['PMX_RECENT'] > 0) ) {
			foreach ($command['author']->data['PMX_RECENT'] as $recent_login) {
				if ($player->login == $recent_login) {
					continue 2;
				}
			}
		}

		// Hide himself from list
		if ($player->login == $command['author']->login) {
			continue 1;
		}

		$all_players[] = array(
			array(pmx_handleNicknames($player->nickname), $pmx_config['MANIALINK_ID'].$player->id),
			'$n'. $player->login,
			$player->isspectator ? 'Spectator' : 'Player'
		);

		$lines++;
		if ($lines > 12) {
			$command['author']->msgs[] = $all_players;
			$lines = 0;
			$all_players = array();
			$all_players = array_merge($title);
		}

		$player_count ++;
	}
	// Add if last batch exists
	if (count($all_players) > 2) {		// two header lines
		$command['author']->msgs[] = $all_players;
	}
	unset($all_players);


//	if ($aseco->server->getGame() == 'TMN') {
//		// Display popup message
//		$aseco->client->query('SendDisplayServerMessageToLogin', $command['author']->login, $command['author']->msgs[1], 'Close', 'Next', 0);
//	}
//	elseif ($aseco->server->getGame() == 'TMF') {
		// Display ManiaLink message
		display_manialink_multi($command['author']);
//	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerManialinkPageAnswer
// $answer[0] = $player->id from sender
// $answer[1] = $player->login from sender
// $answer[2] = ManialinkIdPlayerId to send the message, e.g. 9133541
function pmx_onPlayerManialinkPageAnswer ($aseco, $answer) {
	global $pmx_config;

	if (substr($answer[2], 0, strlen($pmx_config['MANIALINK_ID'])) == $pmx_config['MANIALINK_ID']) {

		// Remove the MANIALINK_ID from target PlayerID
		$target = str_replace($pmx_config['MANIALINK_ID'], '', $answer[2]);

		// Set the author of this action
		$command['author'] = $aseco->server->players->getPlayer($answer[1]);

		foreach ($aseco->server->players->player_list as $player) {
			if ($player->id == $target) {

				// Check if player is already in players recent list
				$recent_player_found = false;
				if (count($command['author']->data['PMX_RECENT'] > 0) ) {
					foreach ($command['author']->data['PMX_RECENT'] as $recent_player) {
						if ($recent_player == $player->login) {
							$recent_player_found = true;
						}
					}
				}
				// Insert new contact into players recent list, if not already included
				if ($recent_player_found == false) {
					array_push($command['author']->data['PMX_RECENT'], $player->login);
				}

				// Close popup window (does not close automatic on player click)
				mainwindow_off($aseco, $command['author']->login);

				if ($pmx_config['LOG_USER_ACTION'] == true) {
					$aseco->console('player {1} used chat command "/pmx {2}" for player {3}',
						$command['author']->login,
						$command['author']->data['PMX_MESSAGE'],
						$player->login
					);
				}

				// Send private message, $command['params'] must looks like "Player(Login|ID) message"
				$command['params'] = $player->login .' '. $command['author']->data['PMX_MESSAGE'];
				chat_pm($aseco, $command);
				unset($command['author']->data['PMX_MESSAGE']);	// Clear message
			}
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerConnect
function pmx_onPlayerConnect ($aseco, $player) {

	// Init Players recent list
	$player->data['PMX_RECENT'] = array();
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function pmx_handleNicknames ($nick) {

	// Remove links, e.g. "$(L|H)[...]...$(L|H)"
	$nick = preg_replace('/\${1}(L|H)\[.*?\](.*?)\$(L|H)/i', '$2', $nick);
	$nick = preg_replace('/\${1}(L|H)\[.*?\](.*?)/i', '$2', $nick);

	// Remove $H (manialink)
	// Remove $L (link)
	$nick = preg_replace('/\${1}[LH]/i', '', $nick);

	// Convert >
	// Convert <
	$nick = str_replace(
			array(
				'>',
				'<'
			),
			array(
				'&gt;',
				'&lt;'
			),
			$nick
	);

	return validateUTF8String($nick);	// validateUTF8String() from basic.inc.php
}

?>
